@@ -1,6 +1,6 @@
 # This file is part of the Trezor project.
 #
-# Copyright (C) 2012-2019 SatoshiLabs and contributors
+# Copyright (C) 2012-2022 SatoshiLabs and contributors
 #
 # This library is free software: you can redistribute it and/or modify
 # it under the terms of the GNU Lesser General Public License version 3
@@ -14,33 +14,46 @@
 # You should have received a copy of the License along with this library.
 # If not, see <https://www.gnu.org/licenses/lgpl-3.0.html>.
 
+from __future__ import annotations
+
 import logging
 import os
 import warnings
-from typing import Optional
+from typing import TYPE_CHECKING, Any, Generic, Optional, TypeVar
 
 from mnemonic import Mnemonic
 
-from . import MINIMUM_FIRMWARE_VERSION, exceptions, mapping, messages, tools
+from . import exceptions, mapping, messages, models
 from .log import DUMP_BYTES
 from .messages import Capability
+from .protobuf import MessageType
+from .tools import parse_path, session
+
+if TYPE_CHECKING:
+    from .transport import Transport
+    from .ui import TrezorClientUI
+
+UI = TypeVar("UI", bound="TrezorClientUI")
+MT = TypeVar("MT", bound=MessageType)
 
 LOG = logging.getLogger(__name__)
 
-VENDORS = ("bitcointrezor.com", "trezor.io")
 MAX_PASSPHRASE_LENGTH = 50
+MAX_PIN_LENGTH = 50
 
 PASSPHRASE_ON_DEVICE = object()
-PASSPHRASE_TEST_PATH = tools.parse_path("44h/1h/0h/0/0")
+PASSPHRASE_TEST_PATH = parse_path("44h/1h/0h/0/0")
 
 OUTDATED_FIRMWARE_ERROR = """
 Your Trezor firmware is out of date. Update it with the following command:
-  trezorctl firmware-update
-Or visit https://wallet.trezor.io/
+  trezorctl firmware update
+Or visit https://suite.trezor.io/
 """.strip()
 
 
-def get_default_client(path=None, ui=None, **kwargs):
+def get_default_client(
+    path: Optional[str] = None, ui: Optional["TrezorClientUI"] = None, **kwargs: Any
+) -> "TrezorClient":
     """Get a client for a connected Trezor device.
 
     Returns a TrezorClient instance with minimum fuss.
@@ -62,94 +75,125 @@ def get_default_client(path=None, ui=None, **kwargs):
     return TrezorClient(transport, ui, **kwargs)
 
 
-class TrezorClient:
+class TrezorClient(Generic[UI]):
     """Trezor client, a connection to a Trezor device.
 
     This class allows you to manage connection state, send and receive protobuf
     messages, handle user interactions, and perform some generic tasks
     (send a cancel message, initialize or clear a session, ping the device).
-
-    You have to provide a transport, i.e., a raw connection to the device. You can use
-    `trezorlib.transport.get_transport` to find one.
-
-    You have to provide an UI implementation for the three kinds of interaction:
-    - button request (notify the user that their interaction is needed)
-    - PIN request (on T1, ask the user to input numbers for a PIN matrix)
-    - passphrase request (ask the user to enter a passphrase)
-    See `trezorlib.ui` for details.
-
-    You can supply a `session_id` you might have saved in the previous session.
-    If you do, the user might not need to enter their passphrase again.
     """
 
+    model: models.TrezorModel
+    transport: "Transport"
+    session_id: Optional[bytes]
+    ui: UI
+    features: messages.Features
+
     def __init__(
-        self, transport, ui, session_id=None,
-    ):
-        LOG.info("creating client instance for device: {}".format(transport.get_path()))
+        self,
+        transport: "Transport",
+        ui: UI,
+        session_id: Optional[bytes] = None,
+        derive_cardano: Optional[bool] = None,
+        model: Optional[models.TrezorModel] = None,
+        _init_device: bool = True,
+    ) -> None:
+        """Create a TrezorClient instance.
+
+        You have to provide a `transport`, i.e., a raw connection to the device. You can
+        use `trezorlib.transport.get_transport` to find one.
+
+        You have to provide a UI implementation for the three kinds of interaction:
+        - button request (notify the user that their interaction is needed)
+        - PIN request (on T1, ask the user to input numbers for a PIN matrix)
+        - passphrase request (ask the user to enter a passphrase) See `trezorlib.ui` for
+          details.
+
+        You can supply a `session_id` you might have saved in the previous session. If
+        you do, the user might not need to enter their passphrase again.
+
+        You can provide Trezor model information. If not provided, it is detected from
+        the model name reported at initialization time.
+
+        By default, the instance will open a connection to the Trezor device, send an
+        `Initialize` message, set up the `features` field from the response, and connect
+        to a session. By specifying `_init_device=False`, this step is skipped. Notably,
+        this means that `client.features` is unset. Use `client.init_device()` or
+        `client.refresh_features()` to fix that, otherwise A LOT OF THINGS will break.
+        Only use this if you are _sure_ that you know what you are doing. This feature
+        might be removed at any time.
+        """
+        LOG.info(f"creating client instance for device: {transport.get_path()}")
+        # Here, self.model could be set to None. Unless _init_device is False, it will
+        # get correctly reconfigured as part of the init_device flow.
+        self.model = model  # type: ignore ["None" is incompatible with "TrezorModel"]
+        if self.model:
+            self.mapping = self.model.default_mapping
+        else:
+            self.mapping = mapping.DEFAULT_MAPPING
         self.transport = transport
         self.ui = ui
-        self.session_id = session_id
         self.session_counter = 0
-        self.init_device()
+        self.session_id = session_id
+        if _init_device:
+            self.init_device(session_id=session_id, derive_cardano=derive_cardano)
 
-    def open(self):
+    def open(self) -> None:
         if self.session_counter == 0:
             self.transport.begin_session()
         self.session_counter += 1
 
-    def close(self):
+    def close(self) -> None:
         self.session_counter = max(self.session_counter - 1, 0)
         if self.session_counter == 0:
             # TODO call EndSession here?
             self.transport.end_session()
 
-    def cancel(self):
+    def cancel(self) -> None:
         self._raw_write(messages.Cancel())
 
-    def call_raw(self, msg):
+    def call_raw(self, msg: MessageType) -> MessageType:
         __tracebackhide__ = True  # for pytest # pylint: disable=W0612
         self._raw_write(msg)
         return self._raw_read()
 
-    def _raw_write(self, msg):
+    def _raw_write(self, msg: MessageType) -> None:
         __tracebackhide__ = True  # for pytest # pylint: disable=W0612
         LOG.debug(
-            "sending message: {}".format(msg.__class__.__name__),
+            f"sending message: {msg.__class__.__name__}",
             extra={"protobuf": msg},
         )
-        msg_type, msg_bytes = mapping.encode(msg)
+        msg_type, msg_bytes = self.mapping.encode(msg)
         LOG.log(
             DUMP_BYTES,
-            "encoded as type {} ({} bytes): {}".format(
-                msg_type, len(msg_bytes), msg_bytes.hex()
-            ),
+            f"encoded as type {msg_type} ({len(msg_bytes)} bytes): {msg_bytes.hex()}",
         )
         self.transport.write(msg_type, msg_bytes)
 
-    def _raw_read(self):
+    def _raw_read(self) -> MessageType:
         __tracebackhide__ = True  # for pytest # pylint: disable=W0612
         msg_type, msg_bytes = self.transport.read()
         LOG.log(
             DUMP_BYTES,
-            "received type {} ({} bytes): {}".format(
-                msg_type, len(msg_bytes), msg_bytes.hex()
-            ),
+            f"received type {msg_type} ({len(msg_bytes)} bytes): {msg_bytes.hex()}",
         )
-        msg = mapping.decode(msg_type, msg_bytes)
+        msg = self.mapping.decode(msg_type, msg_bytes)
         LOG.debug(
-            "received message: {}".format(msg.__class__.__name__),
+            f"received message: {msg.__class__.__name__}",
             extra={"protobuf": msg},
         )
         return msg
 
-    def _callback_pin(self, msg):
+    def _callback_pin(self, msg: messages.PinMatrixRequest) -> MessageType:
         try:
             pin = self.ui.get_pin(msg.type)
         except exceptions.Cancelled:
             self.call_raw(messages.Cancel())
             raise
 
-        if any(d not in "123456789" for d in pin) or not (1 <= len(pin) <= 9):
+        if any(d not in "123456789" for d in pin) or not (
+            1 <= len(pin) <= MAX_PIN_LENGTH
+        ):
             self.call_raw(messages.Cancel())
             raise ValueError("Invalid PIN provided")
 
@@ -163,10 +207,12 @@ class TrezorClient:
         else:
             return resp
 
-    def _callback_passphrase(self, msg: messages.PassphraseRequest):
+    def _callback_passphrase(self, msg: messages.PassphraseRequest) -> MessageType:
         available_on_device = Capability.PassphraseEntry in self.features.capabilities
 
-        def send_passphrase(passphrase=None, on_device=None):
+        def send_passphrase(
+            passphrase: Optional[str] = None, on_device: Optional[bool] = None
+        ) -> MessageType:
             msg = messages.PassphraseAck(passphrase=passphrase, on_device=on_device)
             resp = self.call_raw(msg)
             if isinstance(resp, messages.Deprecated_PassphraseStateRequest):
@@ -192,6 +238,8 @@ class TrezorClient:
                 return send_passphrase(on_device=True)
 
         # else process host-entered passphrase
+        if not isinstance(passphrase, str):
+            raise RuntimeError("Passphrase must be a str")
         passphrase = Mnemonic.normalize_string(passphrase)
         if len(passphrase) > MAX_PASSPHRASE_LENGTH:
             self.call_raw(messages.Cancel())
@@ -199,15 +247,15 @@ class TrezorClient:
 
         return send_passphrase(passphrase, on_device=False)
 
-    def _callback_button(self, msg):
+    def _callback_button(self, msg: messages.ButtonRequest) -> MessageType:
         __tracebackhide__ = True  # for pytest # pylint: disable=W0612
         # do this raw - send ButtonAck first, notify UI later
         self._raw_write(messages.ButtonAck())
-        self.ui.button_request(msg.code)
+        self.ui.button_request(msg)
         return self._raw_read()
 
-    @tools.session
-    def call(self, msg):
+    @session
+    def call(self, msg: MessageType, expect: type[MT] = MessageType) -> MT:
         self.check_firmware_version()
         resp = self.call_raw(msg)
         while True:
@@ -221,13 +269,19 @@ class TrezorClient:
                 if resp.code == messages.FailureType.ActionCancelled:
                     raise exceptions.Cancelled
                 raise exceptions.TrezorFailure(resp)
+            elif not isinstance(resp, expect):
+                raise exceptions.UnexpectedMessageError(expect, resp)
             else:
                 return resp
 
     def _refresh_features(self, features: messages.Features) -> None:
         """Update internal fields based on passed-in Features message."""
-        if features.vendor not in VENDORS:
-            raise RuntimeError("Unsupported device")
+
+        if not self.model:
+            self.model = models.detect(features)
+
+        if features.vendor not in self.model.vendors:
+            raise exceptions.TrezorException(f"Unrecognized vendor: {features.vendor}")
 
         self.features = features
         self.version = (
@@ -240,7 +294,7 @@ class TrezorClient:
             self.session_id = self.features.session_id
             self.features.session_id = None
 
-    @tools.session
+    @session
     def refresh_features(self) -> messages.Features:
         """Reload features from the device.
 
@@ -253,9 +307,13 @@ class TrezorClient:
         self._refresh_features(resp)
         return resp
 
-    @tools.session
+    @session
     def init_device(
-        self, *, session_id: bytes = None, new_session: bool = False
+        self,
+        *,
+        session_id: Optional[bytes] = None,
+        new_session: bool = False,
+        derive_cardano: Optional[bool] = None,
     ) -> Optional[bytes]:
         """Initialize the device and return a session ID.
 
@@ -290,10 +348,23 @@ class TrezorClient:
         elif session_id is not None:
             self.session_id = session_id
 
-        resp = self.call_raw(messages.Initialize(session_id=self.session_id))
+        resp = self.call_raw(
+            messages.Initialize(
+                session_id=self.session_id,
+                derive_cardano=derive_cardano,
+            )
+        )
+        if isinstance(resp, messages.Failure):
+            # can happen if `derive_cardano` does not match the current session
+            raise exceptions.TrezorFailure(resp)
         if not isinstance(resp, messages.Features):
             raise exceptions.TrezorException("Unexpected response to Initialize")
 
+        if self.session_id is not None and resp.session_id == self.session_id:
+            LOG.info("Successfully resumed session")
+        elif session_id is not None:
+            LOG.info("Failed to resume session")
+
         # TT < 2.3.0 compatibility:
         # _refresh_features will clear out the session_id field. We want this function
         # to return its value, so that callers can rely on it being either a valid
@@ -305,24 +376,19 @@ class TrezorClient:
         self._refresh_features(resp)
         return reported_session_id
 
-    def is_outdated(self):
+    def is_outdated(self) -> bool:
         if self.features.bootloader_mode:
             return False
-        model = self.features.model or "1"
-        required_version = MINIMUM_FIRMWARE_VERSION[model]
-        return self.version < required_version
+        return self.version < self.model.minimum_version
 
-    def check_firmware_version(self, warn_only=False):
+    def check_firmware_version(self, warn_only: bool = False) -> None:
         if self.is_outdated():
             if warn_only:
                 warnings.warn("Firmware is out of date", stacklevel=2)
             else:
                 raise exceptions.OutdatedFirmwareError(OUTDATED_FIRMWARE_ERROR)
 
-    @tools.expect(messages.Success, field="message")
-    def ping(
-        self, msg, button_protection=False,
-    ):
+    def ping(self, msg: str, button_protection: bool = False) -> str:
         # We would like ping to work on any valid TrezorClient instance, but
         # due to the protection modes, we need to go through self.call, and that will
         # raise an exception if the firmware is too old.
@@ -336,18 +402,24 @@ class TrezorClient:
                     # device is PIN-locked.
                     # respond and hope for the best
                     resp = self._callback_button(resp)
-                return resp
+                resp = messages.Success.ensure_isinstance(resp)
+                assert resp.message is not None
+                return resp.message
             finally:
                 self.close()
 
-        msg = messages.Ping(message=msg, button_protection=button_protection,)
-        return self.call(msg)
+        resp = self.call(
+            messages.Ping(message=msg, button_protection=button_protection),
+            expect=messages.Success,
+        )
+        assert resp.message is not None
+        return resp.message
 
-    def get_device_id(self):
+    def get_device_id(self) -> Optional[str]:
         return self.features.device_id
 
-    @tools.session
-    def lock(self):
+    @session
+    def lock(self, *, _refresh_features: bool = True) -> None:
         """Lock the device.
 
         If the device does not have a PIN configured, this will do nothing.
@@ -360,11 +432,15 @@ class TrezorClient:
         To invalidate passphrase cache, use `end_session()`. To lock _and_ invalidate
         passphrase cache, use `clear_session()`.
         """
+        # Private argument _refresh_features can be used internally to avoid
+        # refreshing in cases where we will refresh soon anyway. This is used
+        # in TrezorClient.clear_session()
         self.call(messages.LockDevice())
-        self.refresh_features()
+        if _refresh_features:
+            self.refresh_features()
 
-    @tools.session
-    def ensure_unlocked(self):
+    @session
+    def ensure_unlocked(self) -> None:
         """Ensure the device is unlocked and a passphrase is cached.
 
         If the device is locked, this will prompt for PIN. If passphrase is enabled
@@ -379,17 +455,27 @@ class TrezorClient:
         get_address(self, "Testnet", PASSPHRASE_TEST_PATH)
         self.refresh_features()
 
-    def end_session(self):
+    def end_session(self) -> None:
         """Close the current session and clear cached passphrase.
 
         The session will become invalid until `init_device()` is called again.
         If passphrase is enabled, further actions will prompt for it again.
+
+        This is a no-op in bootloader mode, as it does not support session management.
         """
-        # XXX self.call(messages.EndSession())
+        # since: 2.3.4, 1.9.4
+        try:
+            if not self.features.bootloader_mode:
+                self.call(messages.EndSession())
+        except exceptions.TrezorFailure:
+            # A failure most likely means that the FW version does not support
+            # the EndSession call. We ignore the failure and clear the local session_id.
+            # The client-side end result is identical.
+            pass
         self.session_id = None
 
-    @tools.session
-    def clear_session(self):
+    @session
+    def clear_session(self) -> None:
         """Lock the device and present a fresh session.
 
         The current session will be invalidated and a new one will be started. If the
@@ -397,7 +483,6 @@ class TrezorClient:
 
         Equivalent to calling `lock()`, `end_session()` and `init_device()`.
         """
-        # call LockDevice manually to save one refresh_features() call
-        self.call(messages.LockDevice())
+        self.lock(_refresh_features=False)
         self.end_session()
-        self.init_device()
+        self.init_device(new_session=True)
